'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _uniq2 = require('lodash/uniq');

var _uniq3 = _interopRequireDefault(_uniq2);

var _sparkCore = require('@ciscospark/spark-core');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var AvatarUrlBatcher = _sparkCore.Batcher.extend({
  namespace: 'Avatar',

  handleHttpSuccess: function handleHttpSuccess(res) {
    var _this = this;

    // eslint-disable-next-line arrow-body-style
    return _promise2.default.all(res.options.body.map(function (req) {
      return _promise2.default.all(req.sizes.map(function (size) {
        var response = res.body[req.uuid] && res.body[req.uuid][size] || undefined;

        return _this.acceptItem({
          response: response,
          uuid: req.uuid,
          size: size
        });
      }));
    }));
  },
  handleHttpError: function handleHttpError(reason) {
    var _this2 = this;

    var msg = reason.message || reason.body || reason;

    // avoid multiple => on same line
    // eslint-disable-next-line arrow-body-style
    return _promise2.default.all(reason.options.body.map(function (item) {
      return _promise2.default.all(item.sizes.map(function (size) {
        return _this2.getDeferredForRequest({
          uuid: item.uuid,
          size: size
        })
        // I don't see a better way to do this than with an additional nesting
        // eslint-disable-next-line max-nested-callbacks
        .then(function (defer) {
          return defer.reject(msg instanceof Error ? msg : new Error(msg));
        });
      }));
    }));
  },
  didItemFail: function didItemFail(item) {
    if (item.response) {
      if (item.size !== item.response.size) {
        this.logger.warn('Avatar: substituted size "' + item.response.size + '" for "' + item.size + '"');
      }

      return _promise2.default.resolve(false);
    }

    return _promise2.default.resolve(true);
  },
  handleItemFailure: function handleItemFailure(item) {
    return this.getDeferredForRequest(item).then(function (defer) {
      defer.reject(new Error(item.response || 'Failed to retrieve avatar'));
    });
  },
  handleItemSuccess: function handleItemSuccess(item) {
    return this.getDeferredForResponse(item).then(function (defer) {
      return defer.resolve({
        hasDefaultAvatar: item.response.defaultAvatar,
        uuid: item.uuid,
        size: item.size,
        url: item.response.url
      });
    });
  },
  fingerprintRequest: function fingerprintRequest(item) {
    return _promise2.default.resolve(item.uuid + '-' + item.size);
  },
  fingerprintResponse: function fingerprintResponse(item) {
    return _promise2.default.resolve(item.uuid + '-' + item.size);
  },
  prepareRequest: function prepareRequest(queue) {
    var map = queue.reduce(function (m, item) {
      var o = m.get(item.uuid);

      if (!o) {
        o = [];
        m.set(item.uuid, o);
      }
      o.push(item.size);

      return m;
    }, new _map2.default());

    var payload = [];

    map.forEach(function (value, key) {
      payload.push({
        uuid: key,
        sizes: (0, _uniq3.default)(value)
      });
    });

    return _promise2.default.resolve(payload);
  },
  submitHttpRequest: function submitHttpRequest(payload) {
    return this.spark.request({
      method: 'POST',
      api: 'avatar',
      resource: 'profiles/urls',
      body: payload
    });
  }
}); /*!
     * Copyright (c) 2015-2017 Cisco Systems, Inc. See LICENSE file.
     */

exports.default = AvatarUrlBatcher;
//# sourceMappingURL=avatar-url-batcher.js.map
