'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _weakMap = require('babel-runtime/core-js/weak-map');

var _weakMap2 = _interopRequireDefault(_weakMap);

var _common = require('@ciscospark/common');

var _commonTimers = require('@ciscospark/common-timers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * <uuid+size, {uuid, size, url}> map
 */
/*!
 * Copyright (c) 2015-2017 Cisco Systems, Inc. See LICENSE file.
 */

/* eslint-disable complexity */

var urlByUuid = new _weakMap2.default();

/**
 * @class AvatarUrlStore
 */

var AvatarUrlStore = function () {
  /**
   * @constructs {AvatarUrlStore}
   */
  function AvatarUrlStore() {
    (0, _classCallCheck3.default)(this, AvatarUrlStore);

    urlByUuid.set(this, new _map2.default());
  }

  /**
   * Get the URL associated with the given uuid and size.
   *
   * @param {object} item
   * @param {string} item.uuid A user uuid
   * @param {integer}item.size the requested size
   * @returns {Promise<object>} Resolves to the avatar item {uuid, url, size, cacheControl}
   *                           or Rejects on bad param, not in store
   *
   * @memberOf AvatarUrlStore
   */


  (0, _createClass3.default)(AvatarUrlStore, [{
    key: 'get',
    value: function get(item) {
      if (!item) {
        return _promise2.default.reject(new Error('`item` is required'));
      }
      if (!item.uuid) {
        return _promise2.default.reject(new Error('`item.uuid` is required'));
      }
      if (!item.size) {
        return _promise2.default.reject(new Error('`item.size` is required'));
      }
      if (!_common.patterns.uuid.test(item.uuid)) {
        return _promise2.default.reject(new Error('`item.uuid` does not appear to be a uuid'));
      }
      var ret = urlByUuid.get(this).get(item.uuid + ' - ' + item.size);

      if (ret) {
        return _promise2.default.resolve(ret);
      }

      return _promise2.default.reject(new Error('No URL found by specified id: ' + (0, _stringify2.default)(item)));
    }

    /**
     * Adds the given item to the store
     * @param {Object} item
     * @param {integer} item.cacheControl
     * @param {integer} item.hasDefaultAvatar
     * @param {integer} item.size
     * @param {string} item.url
     * @param {string} item.uuid
     * @returns {Promise<object>} Resolves to the added avatar item or rejects on bad params
     */

  }, {
    key: 'add',
    value: function add(item) {
      if (!item) {
        return _promise2.default.reject(new Error('`item` is required'));
      }
      if (!item.uuid) {
        return _promise2.default.reject(new Error('`item.uuid` is required'));
      }
      if (!item.size) {
        return _promise2.default.reject(new Error('`item.size` is required'));
      }
      if (!_common.patterns.uuid.test(item.uuid)) {
        return _promise2.default.reject(new Error('`item.uuid` does not appear to be a uuid'));
      }
      if (!item.url) {
        return _promise2.default.reject(new Error('`item.url` is required'));
      }
      if (!item.cacheControl) {
        return _promise2.default.reject(new Error('`item.cacheControl` is required'));
      }

      (0, _commonTimers.safeSetTimeout)(this.remove.bind(this, item), item.cacheControl * 1000);
      urlByUuid.get(this).set(item.uuid + ' - ' + item.size, item);

      return _promise2.default.resolve(item);
    }

    /**
     * Remove the URL associated with the uuid and size
     * Remove urls of all sizes if size is not given
     *
     * @param {object} item
     * @param {string} item.uuid The user unique id
     * @param {integer} item.size The size of the avatar to remove
     * @returns {Promise<true>}
     */

  }, {
    key: 'remove',
    value: function remove(item) {
      var _this = this;

      var sizes = item.size && [item.size] || [40, 50, 80, 110, 135, 192, 640, 1600];

      sizes.forEach(function (one) {
        return urlByUuid.get(_this).delete(item.uuid + ' - ' + one);
      });

      return _promise2.default.resolve(true);
    }
  }]);
  return AvatarUrlStore;
}();

exports.default = AvatarUrlStore;
//# sourceMappingURL=avatar-url-store.js.map
