'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getOwnPropertyDescriptor = require('babel-runtime/core-js/object/get-own-property-descriptor');

var _getOwnPropertyDescriptor2 = _interopRequireDefault(_getOwnPropertyDescriptor);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _defaults2 = require('lodash/defaults');

var _defaults3 = _interopRequireDefault(_defaults2);

var _dec, _dec2, _desc, _value, _obj; /*!
                                       * Copyright (c) 2015-2017 Cisco Systems, Inc. See LICENSE file.
                                       */

var _avatarUrlBatcher = require('./avatar-url-batcher');

var _avatarUrlBatcher2 = _interopRequireDefault(_avatarUrlBatcher);

var _avatarUrlStore = require('./avatar-url-store');

var _avatarUrlStore2 = _interopRequireDefault(_avatarUrlStore);

var _common = require('@ciscospark/common');

var _helperImage = require('@ciscospark/helper-image');

var _sparkCore = require('@ciscospark/spark-core');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) {
  var desc = {};
  Object['ke' + 'ys'](descriptor).forEach(function (key) {
    desc[key] = descriptor[key];
  });
  desc.enumerable = !!desc.enumerable;
  desc.configurable = !!desc.configurable;

  if ('value' in desc || desc.initializer) {
    desc.writable = true;
  }

  desc = decorators.slice().reverse().reduce(function (desc, decorator) {
    return decorator(target, property, desc) || desc;
  }, desc);

  if (context && desc.initializer !== void 0) {
    desc.value = desc.initializer ? desc.initializer.call(context) : void 0;
    desc.initializer = undefined;
  }

  if (desc.initializer === void 0) {
    Object['define' + 'Property'](target, property, desc);
    desc = null;
  }

  return desc;
}

var Avatar = _sparkCore.SparkPlugin.extend((_dec = (0, _common.oneFlight)({ keyFactory: function keyFactory(uuid) {
    return uuid;
  } }), _dec2 = (0, _common.oneFlight)({ keyFactory: function keyFactory(uuid, options) {
    return uuid + String(options && options.size);
  } }), (_obj = {
  namespace: 'Avatar',

  children: {
    batcher: _avatarUrlBatcher2.default
  },

  session: {
    store: {
      default: function _default() {
        return new _avatarUrlStore2.default();
      },

      type: 'any'
    },
    enableThumbnails: {
      default: true,
      type: 'boolean'
    }
  },

  _fetchAllAvatarUrlSizes: function _fetchAllAvatarUrlSizes(uuid, options) {
    var _this = this;

    // fetch all possible sizes of avatar and store in cache
    return _promise2.default.all(this.config.sizes.map(function (size) {
      return _this.batcher.request({ uuid: uuid, size: size }).then(function (item) {
        return _this.store.add((0, _defaults3.default)({ cacheControl: options.cacheControl }, item));
      });
    }));
  },
  _fetchAvatarUrl: function _fetchAvatarUrl(uuid, options) {
    var _this2 = this;

    return this.store.get({ uuid: uuid, size: options.size }).catch(function () {
      return _promise2.default.all([_this2._fetchAllAvatarUrlSizes(uuid, options),
      // just in case options.size does not fall into the predefined values above
      _this2.batcher.request({ uuid: uuid, size: options.size })])
      // eslint-disable-next-line no-unused-vars
      .then(function (_ref) {
        var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
            ignore = _ref2[0],
            item = _ref2[1];

        return _this2.store.add((0, _defaults3.default)({ cacheControl: options.cacheControl }, item));
      });
    });
  },


  /**
   * Retrieves an Avatar from a cache or the api on misses.
   *
   * @param {UserObject|string} user The user, Spark user uuid, or email
   * @param {Object} [options]
   * @param {integer} [options.size] In {1600, 640, 192, 135, 110, 80, 50, 40}
   *                                 Defaults to 80 if falsy
   * @param {boolean} [options.hideDefaultAvatar] does not return default avatar url if true. Defaults to false
   * @returns {Promise<string>} A promise that resolves to the avatar
   */
  retrieveAvatarUrl: function retrieveAvatarUrl(user, options) {
    var _this3 = this;

    if (!user) {
      return _promise2.default.reject(new Error('\'user\' is a required parameter'));
    }

    options = (0, _defaults3.default)(options, {
      cacheControl: this.config.cacheControl,
      size: this.config.defaultAvatarSize
    });

    return this.spark.internal.user.asUUID(user).then(function (uuid) {
      return _this3._fetchAvatarUrl(uuid, options);
    }).then(function (item) {
      if (options.hideDefaultAvatar) {
        return item.hasDefaultAvatar ? null : item.url;
      }

      return item.url;
    });
  },


  /**
   * Upload a new avatar for the current user
   *
   * @param {Blob|File|Buffer} file The new avatar
   * @returns {Promise} Resolves with the URL of the full-sized avatar
   */
  setAvatar: function setAvatar(file) {
    var _this4 = this;

    return (0, _helperImage.detectFileType)(file, this.logger).then(function (type) {
      return (0, _helperImage.processImage)({
        file: file,
        type: type,
        thumbnailMaxWidth: _this4.config.thumbnailMaxWidth,
        thumbnailMaxHeight: _this4.config.thumbnailMaxHeight,
        enableThumbnails: _this4.enableThumbnails,
        logger: _this4.logger,
        isAvatar: true
      });
    }).then(function (file) {
      return _this4.upload({
        api: 'avatar',
        resource: 'profile',
        file: file[0],
        phases: {
          upload: {
            $uri: function $uri(session) {
              return session.url;
            }
          },
          finalize: {
            method: 'PUT',
            api: 'avatar',
            // eslint-disable-next-line no-unused-vars
            $resource: function $resource(session) {
              return 'profile/' + session.id;
            },
            $body: function $body(session) {
              return session;
            }
          }
        }
      });
    }).then(function (res) {
      // invalidate user's cached avatar
      _this4.store.remove({ uuid: _this4.spark.internal.device.userId });

      return res.url;
    });
  },
  version: '1.52.4'
}, (_applyDecoratedDescriptor(_obj, '_fetchAllAvatarUrlSizes', [_dec], (0, _getOwnPropertyDescriptor2.default)(_obj, '_fetchAllAvatarUrlSizes'), _obj), _applyDecoratedDescriptor(_obj, '_fetchAvatarUrl', [_dec2], (0, _getOwnPropertyDescriptor2.default)(_obj, '_fetchAvatarUrl'), _obj)), _obj)));

exports.default = Avatar;
//# sourceMappingURL=avatar.js.map
